package cn.com.qmth.scancloud.tools.service.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.scancloud.tools.config.SysProperty;
import cn.com.qmth.scancloud.tools.enums.TaskType;
import cn.com.qmth.scancloud.tools.model.ExamStudent;
import cn.com.qmth.scancloud.tools.service.AbstractTask;
import cn.com.qmth.scancloud.tools.service.CommonService;
import cn.com.qmth.scancloud.tools.utils.FileHelper;
import cn.com.qmth.scancloud.tools.utils.HttpHelper;
import cn.com.qmth.scancloud.tools.utils.JsonHelper;
import cn.com.qmth.scancloud.tools.utils.StatusException;

/**
 * 缺考导入
 */
public class AbsentImportTask extends AbstractTask {

    private static final Logger log = LoggerFactory.getLogger(AbsentImportTask.class);

    @Override
    protected String getTaskName() {
        return TaskType.ABSENT_IMPORT.getTitle();
    }

    @Override
    protected void execute() {
        Long examId = SysProperty.EXAM_ID;
        if (examId == null) {
            throw new StatusException("【scan.tool.examId】未配置！");
        }

        if (CommonService.findExam(examId) == null) {
            throw new StatusException("当前考试不存在！examId = " + examId);
        }

        if (!SysProperty.ABSENT_IMPORT.endsWith(".txt")) {
            throw new StatusException("导入模板目前仅支持后缀名为“.txt”的文件！");
        }

        int total = 0;
        List<ExamStudent> list = new ArrayList<>();
        Set<String> examNumbers = new HashSet<>();

        File file = new File(SysProperty.DATA_DIR + "/" + SysProperty.ABSENT_IMPORT);
        List<String> lines = FileHelper.readAllLines(file);
        for (int n = 1; n < lines.size(); n++) {

            ExamStudent data = this.parseValues(total, lines.get(n), examNumbers);
            if (data != null) {
                data.setExamId(examId);
                list.add(data);

                // 同个考试下考生的“准考证号”值唯一
                examNumbers.add(data.getExamNumber());
            }

            total++;
        }

        log.info("导入文件全部行数共 {} 行，实际有效数据共 {} 条！", total, list.size());
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        // 分批保存
        int batchSize = 5000;
        List<ExamStudent> batchList = new ArrayList<>();
        String url = SysProperty.SCAN_SERVER_URL + "/api/tool/import/cet/absent";
        for (int n = 0; n < list.size(); n++) {
            batchList.add(list.get(n));

            if (batchList.size() % batchSize == 0) {
                String json = JsonHelper.toJson(batchList);
                String result = HttpHelper.post(url, json);
                batchList.clear();

                float rate = (n + 1) * 100f / list.size();
                log.info("已处理数：{} 进度：{}% {}", n + 1, rate, result);
            }
        }

        if (CollectionUtils.isNotEmpty(batchList)) {
            String json = JsonHelper.toJson(batchList);
            String result = HttpHelper.post(url, json);
            log.info("已处理数：{} 进度：100% {}", list.size(), result);
        }
    }

    private ExamStudent parseValues(int index, String line, Set<String> examNumbers) {
        if (StringUtils.isBlank(line)) {
            return null;
        }

        line = StringUtils.deleteWhitespace(line);
        String[] values = StringUtils.splitByWholeSeparatorPreserveAllTokens(line, SysProperty.TEMPLATE_SEPARATOR);

        try {

            String examNumber = values[0].trim();
            String absent = values[2].trim();
            String breachCode = values[3].trim();
            if (StringUtils.isEmpty(examNumber)) {
                throw new StatusException(String.format("【第%s行】“准考证号”字段不能为空！ %s", index + 1, line));
            }
            if (examNumbers.contains(examNumber)) {
                throw new StatusException(String.format("【第%s行】考生信息存在重复！ %s", index + 1, line));
            }
            if (StringUtils.isEmpty(absent)) {
                throw new StatusException(String.format("【第%s行】“缺考”字段不能为空！ %s", index + 1, line));
            }
            if (!"0".equals(absent)&&!"1".equals(absent)) {
                throw new StatusException(String.format("【第%s行】“缺考”字段只能是0或1！ %s", index + 1, line));
            }
            if (StringUtils.isEmpty(breachCode)) {
                throw new StatusException(String.format("【第%s行】“违纪”字段不能为空！ %s", index + 1, line));
            }
            if("1".equals(absent)&&!"0".equals(breachCode)) {
                throw new StatusException(String.format("【第%s行】缺考的违纪只能是0！ %s", index + 1, line));
            }
            ExamStudent data = new ExamStudent();
            data.setExamNumber(examNumber);
            data.setAbsent("1".equals(absent) ? true : false);
            data.setBreachCode(breachCode);
            return data;
        } catch (ArrayIndexOutOfBoundsException e) {
            log.error("【第{}行】内容格式错误！ {} {}", index + 1, line, e.toString());
            throw new StatusException("数据内容有误！");
        }
    }


}
