package cn.com.qmth.scancloud.tools.service.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.scancloud.tools.config.SysProperty;
import cn.com.qmth.scancloud.tools.enums.ExamMode;
import cn.com.qmth.scancloud.tools.enums.ImageTransferMode;
import cn.com.qmth.scancloud.tools.enums.TaskType;
import cn.com.qmth.scancloud.tools.model.Exam;
import cn.com.qmth.scancloud.tools.service.AbstractTask;
import cn.com.qmth.scancloud.tools.utils.FileHelper;
import cn.com.qmth.scancloud.tools.utils.HttpHelper;
import cn.com.qmth.scancloud.tools.utils.JsonHelper;
import cn.com.qmth.scancloud.tools.utils.StatusException;

/**
 * 考试导入
 */
public class ExamImportTask extends AbstractTask {

    private static final Logger log = LoggerFactory.getLogger(ExamImportTask.class);

    @Override
    protected String getTaskName() {
        return TaskType.EXAM_IMPORT.getTitle();
    }

    @Override
    protected void execute() {
        Long schoolId = SysProperty.SCHOOL_ID;
        if (schoolId == null) {
            throw new StatusException("【scan.tool.schoolId】未配置！");
        }
        String schoolName = SysProperty.SCHOOL_NAME;
        if (StringUtils.isBlank(schoolName)) {
            throw new StatusException("【scan.tool.schoolName】未配置！");
        }
        if (!SysProperty.EXAM_IMPORT.endsWith(".txt")) {
            throw new StatusException("导入模板目前仅支持后缀名为“.txt”的文件！");
        }

        File file = new File(SysProperty.DATA_DIR + "/" + SysProperty.EXAM_IMPORT);
        List<String> lines = FileHelper.readAllLines(file);
        if (CollectionUtils.isEmpty(lines)) {
            log.warn("导入文件内无数据！");
            return;
        }

        List<Exam> list = new ArrayList<>();
        for (int n = 1; n < lines.size(); n++) {
            Exam data = this.parseValues(n, lines.get(n), SysProperty.TEMPLATE_SEPARATOR);
            if (data != null) {
                data.setSchoolId(schoolId);
                data.setSchoolName(schoolName);
                list.add(data);
            }
        }

        log.info("导入文件全部行数共 {} 行，实际有效数据共 {} 条！", lines.size(), list.size()-1);
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        String url = SysProperty.SCAN_SERVER_URL + "/api/tool/import/exam";
        for (int n = 0; n < list.size(); n++) {
            String json = JsonHelper.toJson(list.get(n));
            String result = HttpHelper.post(url, json);

            float rate = (n + 1) * 100f / list.size();
            log.info("已处理数：{} 进度：{}% {}", n + 1, rate, result);
        }
    }

    private Exam parseValues(int index, String line, String templateSeparator) {
        if (StringUtils.isBlank(line)) {
            return null;
        }

        line = StringUtils.deleteWhitespace(line);
        String[] values = StringUtils.splitByWholeSeparatorPreserveAllTokens(line, templateSeparator);

        try {
            Long examId = null;
            String examName = null;

            // 默认模板：考试ID|考试名称
            examId = this.parseLong(values[0]);
            examName = values[1];
            String paperTypeCodes = values[2];

            if (examId == null) {
                throw new StatusException(String.format("【第%s行】“考试ID”字段值有误！ %s", index + 1, line));
            }
            if (StringUtils.isEmpty(examName)) {
                throw new StatusException(String.format("【第%s行】“考试名称”字段不能为空！ %s", index + 1, line));
            }
            if (StringUtils.isEmpty(paperTypeCodes)) {
                throw new StatusException(String.format("【第%s行】“试卷条码列表”字段不能为空！ %s", index + 1, line));
            }

            Exam data = new Exam();
            data.setId(examId);
            data.setName(examName);
            data.setImageTransferMode(ImageTransferMode.CET);
            data.setMode(ExamMode.CET);
            data.setScanByPackage(true);
            data.setEnableSyncVerify(true);
            data.setAllowUnexistPaper(false);
            data.setAnswerPaperNumberFigure(0);
            data.setAnswerFrontCardType(1);
            data.setEnableSinglePageAnswer(false);

            data.setPaperTypeBarcodeContent(Arrays.asList(paperTypeCodes.split(";")));

            for (String barcode : data.getPaperTypeBarcodeContent()) {
                if (StringUtils.length(barcode) != 6 || parseLong(barcode) == null) {
                    throw new StatusException("“试卷条码列表”值必须为6位数字！");
                }
            }

            if (data.getAbsentBarcodeContent() == null) {
                data.setAbsentBarcodeContent("");
            }

            if (data.getPaperTypeBarcodeContent() == null) {
                data.setPaperTypeBarcodeContent(new ArrayList<>());
            }

            return data;
        } catch (ArrayIndexOutOfBoundsException e) {
            log.error("【第{}行】内容格式错误！ {} {}", index + 1, line, e.toString());
            throw new StatusException("数据内容有误！");
        }
    }

    private Long parseLong(String str) {
        try {
            return Long.parseLong(str);
        } catch (Exception e) {
            return null;
        }
    }

}
