package cn.com.qmth.scancloud.tools.service.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.JsonNode;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;

import cn.com.qmth.scancloud.tools.config.SysProperty;
import cn.com.qmth.scancloud.tools.enums.ExamStatus;
import cn.com.qmth.scancloud.tools.enums.TaskType;
import cn.com.qmth.scancloud.tools.model.CetMarking;
import cn.com.qmth.scancloud.tools.model.ExportCetVo;
import cn.com.qmth.scancloud.tools.service.AbstractTask;
import cn.com.qmth.scancloud.tools.utils.FileHelper;
import cn.com.qmth.scancloud.tools.utils.HttpHelper;
import cn.com.qmth.scancloud.tools.utils.JsonHelper;
import cn.com.qmth.scancloud.tools.utils.StatusException;

/**
 * 考生扫描结果和评卷数据导出
 */
public class ObjectiveQuestionExportTask extends AbstractTask {

    private static final Logger log = LoggerFactory.getLogger(ObjectiveQuestionExportTask.class);

    private static final String[] EXCEL_HEADER = new String[] { "科目名称", "科目代码", "试卷类型", "条形码值", "奇数考场评卷点", "奇数考场评卷点代码",
            "偶数考场评卷点", "偶数考场评卷点代码" };

    @Override
    protected String getTaskName() {
        return TaskType.OBJECTIVE_QUESTION_EXPORT.getTitle();
    }

    @Override
    protected void execute() {
        Long examId = SysProperty.EXAM_ID;
        if (examId == null) {
            throw new StatusException("【scan.tool.examId】未配置！");
        }

        Map<String, Object> params = new HashMap<>();
        params.put("examId", examId);

        String url = SysProperty.SCAN_SERVER_URL + "/api/tool/exam/student/count";
        String json = JsonHelper.toJson(params);
        String result = HttpHelper.post(url, json);
        log.info("查询考生参数：{}，结果：{}", json, result);

        int total = 0;
        JsonNode props = JsonHelper.getNode(result);
        if (props != null && props.has("count")) {
            total = props.get("count").asInt();
        }
        if (total == 0) {
            log.warn("未查询到相关考生！");
            return;
        }
        Map<String, CetMarking> cms = readCetMarking();
        // 分批处理
        String filePath = String.format("%s/export-slice-data.txt", SysProperty.DATA_DIR);
        String answerFilePath = String.format("%s/export-answer-data.txt", SysProperty.DATA_DIR);
        String queryUrl = SysProperty.SCAN_SERVER_URL + "/api/tool/export/cet/data";
        params.put("pageSize", 100);
        int pageNumber = 0;
        int sum = 0;
        Set<String> examNumbers = new HashSet<>();
        while (true) {
            params.put("pageNumber", ++pageNumber);
            String queryJson = JsonHelper.toJson(params);
            String queryResult = HttpHelper.post(queryUrl, queryJson);

            List<ExportCetVo> list = JsonHelper.toList(queryResult, ExportCetVo.class);
            if (CollectionUtils.isEmpty(list)) {
                break;
            }

            exportMarking(list, filePath, cms, examNumbers);
            exportAnswer(list, answerFilePath, cms);
            sum += list.size();
            float rate = sum * 100f / total;
            log.info("已处理数：{} 进度：{}%", sum, rate);
        }

        log.info("导出文件：" + filePath);
    }

    private void exportAnswer(List<ExportCetVo> list, String filePath, Map<String, CetMarking> cms) {
        List<String> lines = new ArrayList<>();
        for (ExportCetVo data : list) {
            List<String> line = new ArrayList<>();
            line.add(data.getExamNumber());
            line.add(data.getName());
            if (ExamStatus.ABSENT.equals(data.getExamStatus())) {
                line.add("1");
            } else {
                line.add("0");
            }
            line.add(data.getSubjectCode());
            line.add(data.getAnswer());
            line.add(data.getCardFirst() + "");
            line.add(data.getCardSecond() + "");
            line.add(StringUtils.isBlank(data.getBreachCode()) ? "0" : data.getBreachCode());
            line.add(data.getPaperType());
            int subjectCode = Integer.valueOf(data.getExamNumber().substring(9, 10));
            String markingCode;
            if (subjectCode >= 3 && subjectCode <= 9) {
                markingCode = "88";
            } else {
                if (ExamStatus.ABSENT.equals(data.getExamStatus()) || "000000".equals(data.getPaperType())
                        || "999999".equals(data.getPaperType())) {
                    markingCode = "00";
                } else {
                    int site = Integer.valueOf(data.getExamNumber().substring(10, 13));
                    CetMarking cm = cms.get(data.getPaperType());
                    if (cm == null) {
                        throw new StatusException("未找到考点数据,PaperType:" + data.getPaperType());
                    }
                    if (site % 2 == 0) {
                        markingCode = cm.getEvenNumber();
                    } else {
                        markingCode = cm.getOddNumber();
                    }
                }
            }
            line.add(markingCode);
            lines.add(StringUtils.join(line, ","));
        }

        File file = new File(filePath);
        if (file.exists()) {
            file.delete();
        }
        FileHelper.writeLines(file, lines, true);
    }

    private void exportMarking(List<ExportCetVo> list, String filePath, Map<String, CetMarking> cms,
            Set<String> examNumbers) {
        String template = "%s,%s,%s";

        List<String> lines = new ArrayList<>();
        for (ExportCetVo data : list) {
            if (examNumbers.contains(data.getExamNumber())) {
                log.warn("有重复数据,准考证号：{}", data.getExamNumber());
                continue;
            }
            examNumbers.add(data.getExamNumber());
            String markingCode;
            if ("888888".equals(data.getPaperType())) {
                markingCode = "88";
            } else if (ExamStatus.ABSENT.equals(data.getExamStatus()) || "000000".equals(data.getPaperType())
                    || "999999".equals(data.getPaperType())) {
                markingCode = "00";
            } else {
                int site = Integer.valueOf(data.getExamNumber().substring(10, 13));
                if (site % 2 == 0) {
                    try {
                        markingCode = cms.get(data.getPaperType()).getEvenNumber();
                    } catch (Exception e) {
                        throw e;
                    }
                } else {
                    markingCode = cms.get(data.getPaperType()).getOddNumber();
                }
            }
            lines.add(String.format(template, data.getExamNumber(), markingCode,
                    StringUtils.join(data.getSliceImageInfo(), ";")));
        }

        File file = new File(filePath);
        if (file.exists()) {
            file.delete();
        }
        FileHelper.writeLines(file, lines, true);
    }

    private Map<String, CetMarking> readCetMarking() {
        if (StringUtils.isBlank(SysProperty.MARKING_PLACE)) {
            throw new StatusException("【scan.tool.marking-place】未配置！");
        }
        File file = new File(SysProperty.DATA_DIR + "/" + SysProperty.MARKING_PLACE);
        if (!file.exists()) {
            throw new StatusException("文件不存在：" + SysProperty.MARKING_PLACE);
        }
        List<CetMarking> list = new ArrayList<CetMarking>();
        InputStream inputStream = null;
        try {
            inputStream = new FileInputStream(file);
            ExcelReader reader = ExcelReader.create(ExcelType.XLSX, inputStream, 0);
            List<DataMap> lineList = reader.getDataMapList();
            if (!Arrays.equals(EXCEL_HEADER, reader.getColumnNames())) {
                throw new StatusException(SysProperty.MARKING_PLACE + "表头错误");
            }
            if (CollectionUtils.isEmpty(lineList)) {
                throw new StatusException(SysProperty.MARKING_PLACE + "无内容");
            }
            for (int i = 0; i < lineList.size(); i++) {
                DataMap line = lineList.get(i);

                CetMarking imp = new CetMarking();
                String paperType = trimAndNullIfBlank(line.get(EXCEL_HEADER[3]));
                if (StringUtils.isBlank(paperType)) {
                    throw new StatusException(newError(i + 2, "条形码值不能为空"));
                }
                imp.setPaperType(paperType);

                String oddNumber = trimAndNullIfBlank(line.get(EXCEL_HEADER[5]));
                if (StringUtils.isBlank(oddNumber)) {
                    throw new StatusException(newError(i + 2, "奇数考场评卷点代码不能为空"));
                }
                imp.setOddNumber(oddNumber);

                String evenNumber = trimAndNullIfBlank(line.get(EXCEL_HEADER[7]));
                if (StringUtils.isBlank(evenNumber)) {
                    throw new StatusException(newError(i + 2, "偶数考场评卷点代码不能为空"));
                }
                imp.setEvenNumber(evenNumber);
                list.add(imp);

            }
            if (list.size() == 0) {
                throw new StatusException("文件内容为空：" + SysProperty.MARKING_PLACE);
            }
            Map<String, CetMarking> map = list.stream()
                    .collect(Collectors.toMap(CetMarking::getPaperType, c -> c, (key1, key2) -> key2));

            return map;
        } catch (Exception e) {
            throw new RuntimeException("系统错误", e);
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                }
            }
        }
    }

    private String trimAndNullIfBlank(String s) {
        if (StringUtils.isBlank(s)) {
            return null;
        }
        return s.trim();
    }

    private String newError(int lineNum, String msg) {
        return SysProperty.MARKING_PLACE + "第" + lineNum + "行" + msg;
    }
}
