package cn.com.qmth.scancloud.tools.service.impl;

import cn.com.qmth.scancloud.tools.config.SysProperty;
import cn.com.qmth.scancloud.tools.enums.TaskType;
import cn.com.qmth.scancloud.tools.model.ExamStudent;
import cn.com.qmth.scancloud.tools.service.AbstractTask;
import cn.com.qmth.scancloud.tools.utils.HttpHelper;
import cn.com.qmth.scancloud.tools.utils.JsonHelper;
import cn.com.qmth.scancloud.tools.utils.StatusException;
import com.fasterxml.jackson.databind.JsonNode;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletionService;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 扫描图片检查&修复
 */
public class ScanImageCheckTask extends AbstractTask {

    private static final Logger log = LoggerFactory.getLogger(ScanImageCheckTask.class);

    @Override
    protected String getTaskName() {
        return TaskType.SCAN_IMAGE_CHECK.getTitle();
    }

    private ExecutorService threadPool = Executors.newFixedThreadPool(SysProperty.THREAD_SIZE);

    private CompletionService<Boolean> threadService = new ExecutorCompletionService<>(threadPool);

    @Override
    protected void execute() {
        Long examId = SysProperty.EXAM_ID;
        if (examId == null) {
            throw new StatusException("【scan.tool.examId】未配置！");
        }

        // 查询参数：{"examId":null,"subjectCode":"","examNumber":"","name":"","studentCode":"","packageCode":"",
        // "seatNumber":"","campusName":"","examSite":"","examRoom":""}
        Map<String, Object> params = new HashMap<>();
        params.put("examId", examId);
        params.put("scanStatus", "SCANNED");
        params.put("withAnswer", false);

        String url = SysProperty.SCAN_SERVER_URL + "/api/tool/exam/student/count";
        String json = JsonHelper.toJson(params);
        String result = HttpHelper.post(url, json);
        log.info("查询考生参数：{}，结果：{}", json, result);

        int total = 0;
        JsonNode props = JsonHelper.getNode(result);
        if (props != null && props.has("count")) {
            total = props.get("count").asInt();
        }
        if (total == 0) {
            log.warn("未查询到相关考生！");
            return;
        }

        // 分批处理
        String queryUrl = SysProperty.SCAN_SERVER_URL + "/api/tool/exam/student/find";
        params.put("pageSize", 100);
        int pageNumber = 0;
        int sum = 0, err = 0;

        while (true) {
            params.put("pageNumber", ++pageNumber);
            String queryJson = JsonHelper.toJson(params);
            String queryResult = HttpHelper.post(queryUrl, queryJson);

            List<ExamStudent> list = JsonHelper.toList(queryResult, ExamStudent.class);
            if (CollectionUtils.isEmpty(list)) {
                break;
            }

            err += this.check(list);
            sum += list.size();

            float rate = sum * 100f / total;
            log.info("已处理数：{} 异常数：{} 进度：{}%", sum, err, rate);
        }

        threadPool.shutdown();
    }

    private int check(List<ExamStudent> list) {
        for (ExamStudent data : list) {
            String url = SysProperty.SCAN_SERVER_URL + "/api/tool/exam/student/check?studentId=" + data.getId();
            threadService.submit(() -> {
                try {
                    HttpHelper.post(url, null);
                    // log.info("科目代码={}, 准考证号={}, 考生编号={}, ID={}, 扫描图片正常！", data.getSubjectCode(), data.getExamNumber(), data.getStudentCode(), data.getId());
                    return true;
                } catch (Exception e) {
                    log.error("科目代码={}, 准考证号={}, 考生编号={}, ID={}, 扫描图片异常！", data.getSubjectCode(), data.getExamNumber(), data.getStudentCode(), data.getId());
                    return false;
                }
            });
        }

        int err = 0;
        for (int n = 0; n < list.size(); n++) {
            try {
                err += threadService.take().get() ? 0 : 1;
            } catch (Exception e) {
                log.error(e.getMessage());
            }
        }

        return err;
    }

}
